@@ -78,7 +78,7 @@ struct liq_image {
     liq_image_get_rgba_row_callback *row_callback;
     void *row_callback_user_info;
     float min_opaque_val;
-    bool free_rows, free_pixels;
+    bool free_pixels, free_rows, free_rows_internal;
 };
 
 typedef struct liq_remapping_result {
@@ -135,12 +135,16 @@ static void liq_verbose_printf(const liq_attr *context, const char *fmt, ...)
 
 inline static void verbose_print(const liq_attr *attr, const char *msg)
 {
-    if (attr->log_callback) attr->log_callback(attr, msg, attr->log_callback_user_info);
+    if (attr->log_callback) {
+        attr->log_callback(attr, msg, attr->log_callback_user_info);
+    }
 }
 
 static void liq_verbose_printf_flush(liq_attr *attr)
 {
-    if (attr->log_flush_callback) attr->log_flush_callback(attr, attr->log_flush_callback_user_info);
+    if (attr->log_flush_callback) {
+        attr->log_flush_callback(attr, attr->log_flush_callback_user_info);
+    }
 }
 
 #if USE_SSE
@@ -160,7 +164,9 @@ inline static bool is_sse2_available()
 LIQ_EXPORT bool liq_crash_if_invalid_handle_pointer_given(const liq_attr *user_supplied_pointer, const char *const expected_magic_header) NEVER_INLINE;
 LIQ_EXPORT bool liq_crash_if_invalid_handle_pointer_given(const liq_attr *user_supplied_pointer, const char *const expected_magic_header)
 {
-    if (!user_supplied_pointer) return false;
+    if (!user_supplied_pointer) {
+        return false;
+    }
 
     if (user_supplied_pointer->magic_header == liq_freed_magic) {
         fprintf(stderr, "%s used after being freed", expected_magic_header);
@@ -176,7 +182,9 @@ LIQ_EXPORT bool liq_crash_if_invalid_handle_pointer_given(const liq_attr *user_s
 LIQ_EXPORT bool liq_crash_if_invalid_pointer_given(void *pointer) NEVER_INLINE;
 LIQ_EXPORT bool liq_crash_if_invalid_pointer_given(void *pointer)
 {
-    if (!pointer) return false;
+    if (!pointer) {
+        return false;
+    }
     // Force a read from the given (potentially invalid) memory location in order to check early whether this crashes the program or not.
     // It doesn't matter what value is read, the code here is just to shut the compiler up about unused read.
     char test_access = *((volatile char *)pointer);
@@ -242,8 +250,8 @@ static unsigned int mse_to_quality(double mse)
     int i;
     /* Allow for floating point errors. */
     double eps = 0.00001;
-
     for(i=100; i > 0; i--) {
+	printf ("%g %g\n", mse, quality_to_mse (i));
         if (mse <= quality_to_mse(i) + eps) {
             return i;
         }
@@ -261,11 +269,14 @@ LIQ_EXPORT liq_error liq_set_quality(liq_attr* attr, int minimum, int target)
     return LIQ_OK;
 }
 
-LIQ_EXPORT liq_error liq_get_quality(liq_attr* attr, int * minimum, int * target)
+LIQ_EXPORT liq_error liq_get_quality(liq_attr *attr, int *minimum, int *target)
 {
     if (!CHECK_STRUCT_TYPE(attr, liq_attr)) return LIQ_INVALID_POINTER;
-    * minimum = mse_to_quality (attr->max_mse);
-    * target = mse_to_quality (attr->target_mse);
+    if (!CHECK_USER_POINTER(minimum)) return LIQ_INVALID_POINTER;
+    if (!CHECK_USER_POINTER(target)) return LIQ_INVALID_POINTER;
+
+    *minimum = mse_to_quality(attr->max_mse);
+    *target = mse_to_quality(attr->target_mse);
     return LIQ_OK;
 }
 
@@ -278,14 +289,23 @@ LIQ_EXPORT liq_error liq_set_max_colors(liq_attr* attr, int colors)
     return LIQ_OK;
 }
 
-LIQ_EXPORT liq_error liq_get_max_colors_2(liq_attr* attr, int * colors)
+LIQ_EXPORT int liq_get_max_colors(liq_attr *attr)
+{
+    if (!CHECK_STRUCT_TYPE(attr, liq_attr)) return -1;
+
+    return attr->max_colors;
+}
+
+LIQ_EXPORT liq_error liq_get_max_colors_2(liq_attr *attr, int * max_colors)
 {
     if (!CHECK_STRUCT_TYPE(attr, liq_attr)) return LIQ_INVALID_POINTER;
-    * colors = attr->max_colors;
+
+    * max_colors = attr->max_colors;
     return LIQ_OK;
 }
 
-LIQ_EXPORT liq_error liq_set_min_posterization(liq_attr* attr, int bits) {
+LIQ_EXPORT liq_error liq_set_min_posterization(liq_attr *attr, int bits)
+{
     if (!CHECK_STRUCT_TYPE(attr, liq_attr)) return LIQ_INVALID_POINTER;
     if (bits < 0 || bits > 4) return LIQ_VALUE_OUT_OF_RANGE;
 
@@ -293,12 +313,21 @@ LIQ_EXPORT liq_error liq_set_min_posterization(liq_attr* attr, int bits) {
     return LIQ_OK;
 }
 
-LIQ_EXPORT liq_error liq_get_min_posterization_2(liq_attr* attr, int * bits) {
+LIQ_EXPORT int liq_get_min_posterization(liq_attr *attr)
+{
+    if (!CHECK_STRUCT_TYPE(attr, liq_attr)) return -1;
+
+    return attr->min_posterization_output;
+}
+
+LIQ_EXPORT liq_error liq_get_min_posterization_2(liq_attr *attr, int *bits)
+{
     if (!CHECK_STRUCT_TYPE(attr, liq_attr)) return LIQ_INVALID_POINTER;
-    * bits = attr->min_posterization_output;
+    *bits = attr->min_posterization_output;
     return LIQ_OK;
 }
 
+
 LIQ_EXPORT liq_error liq_set_speed(liq_attr* attr, int speed)
 {
     if (!CHECK_STRUCT_TYPE(attr, liq_attr)) return LIQ_INVALID_POINTER;
@@ -318,12 +347,20 @@ LIQ_EXPORT liq_error liq_set_speed(liq_attr* attr, int speed)
     return LIQ_OK;
 }
 
-LIQ_EXPORT liq_error liq_get_speed_2(liq_attr* attr, int * speed)
+LIQ_EXPORT int liq_get_speed(liq_attr *attr)
+{
+    if (!CHECK_STRUCT_TYPE(attr, liq_attr)) return -1;
+
+    return attr->speed;
+}
+
+LIQ_EXPORT liq_error liq_get_speed_2(liq_attr *attr, int *speed)
 {
     if (!CHECK_STRUCT_TYPE(attr, liq_attr)) return LIQ_INVALID_POINTER;
-    * speed = attr->speed;
+    *speed = attr->speed;
     return LIQ_OK;
 }
+
 LIQ_EXPORT liq_error liq_set_output_gamma(liq_result* res, double gamma)
 {
     if (!CHECK_STRUCT_TYPE(res, liq_result)) return LIQ_INVALID_POINTER;
@@ -347,13 +384,21 @@ LIQ_EXPORT liq_error liq_set_min_opacity(liq_attr* attr, int min)
     return LIQ_OK;
 }
 
-LIQ_EXPORT liq_error liq_get_min_opacity_2(liq_attr* attr, int * min)
+LIQ_EXPORT int liq_get_min_opacity(liq_attr *attr)
+{
+    if (!CHECK_STRUCT_TYPE(attr, liq_attr)) return -1;
+
+    return MIN(255, 256.0 * attr->min_opaque_val);
+}
+
+LIQ_EXPORT liq_error liq_get_min_opacity_2(liq_attr *attr, int *min)
 {
     if (!CHECK_STRUCT_TYPE(attr, liq_attr)) return LIQ_INVALID_POINTER;
-    * min = (int) (255.0 * attr->min_opaque_val + 0.5);
+    *min = MIN(255, 256.0 * attr->min_opaque_val);
     return LIQ_OK;
 }
 
+
 LIQ_EXPORT void liq_set_last_index_transparent(liq_attr* attr, int is_last)
 {
     if (!CHECK_STRUCT_TYPE(attr, liq_attr)) return;
@@ -385,7 +430,9 @@ LIQ_EXPORT liq_attr* liq_attr_create()
 
 LIQ_EXPORT void liq_attr_destroy(liq_attr *attr)
 {
-    if (!CHECK_STRUCT_TYPE(attr, liq_attr)) return;
+    if (!CHECK_STRUCT_TYPE(attr, liq_attr)) {
+        return;
+    }
 
     liq_verbose_printf_flush(attr);
 
@@ -395,7 +442,9 @@ LIQ_EXPORT void liq_attr_destroy(liq_attr *attr)
 
 LIQ_EXPORT liq_attr* liq_attr_copy(liq_attr *orig)
 {
-    if (!CHECK_STRUCT_TYPE(orig, liq_attr)) return NULL;
+    if (!CHECK_STRUCT_TYPE(orig, liq_attr)) {
+        return NULL;
+    }
 
     liq_attr *attr = orig->malloc(sizeof(liq_attr));
     if (!attr) return NULL;
@@ -403,6 +452,28 @@ LIQ_EXPORT liq_attr* liq_attr_copy(liq_attr *orig)
     return attr;
 }
 
+static void *liq_aligned_malloc(size_t size)
+{
+    unsigned char *ptr = malloc(size + 16);
+    if (!ptr) {
+        return NULL;
+    }
+
+    uintptr_t offset = 16 - ((uintptr_t)ptr & 15); // also reserves 1 byte for ptr[-1]
+    ptr += offset;
+    assert(0 == (((uintptr_t)ptr) & 15));
+    ptr[-1] = offset ^ 0x59; // store how much pointer was shifted to get the original for free()
+    return ptr;
+}
+
+static void liq_aligned_free(void *inptr)
+{
+    unsigned char *ptr = inptr;
+    size_t offset = ptr[-1] ^ 0x59;
+    assert(offset > 0 && offset <= 16);
+    free(ptr - offset);
+}
+
 LIQ_EXPORT liq_attr* liq_attr_create_with_allocator(void* (*custom_malloc)(size_t), void (*custom_free)(void*))
 {
 #if USE_SSE
@@ -411,8 +482,8 @@ LIQ_EXPORT liq_attr* liq_attr_create_with_allocator(void* (*custom_malloc)(size_
     }
 #endif
     if (!custom_malloc && !custom_free) {
-        custom_malloc = malloc;
-        custom_free = free;
+        custom_malloc = liq_aligned_malloc;
+        custom_free = liq_aligned_free;
     } else if (!custom_malloc != !custom_free) {
         return NULL; // either specify both or none
     }
@@ -545,6 +616,7 @@ LIQ_EXPORT liq_error liq_image_set_memory_ownership(liq_image *img, int ownershi
     }
 
     if (ownership_flags & LIQ_OWN_ROWS) {
+        if (img->free_rows_internal) return LIQ_VALUE_OUT_OF_RANGE;
         img->free_rows = true;
     }
 
@@ -554,8 +626,7 @@ LIQ_EXPORT liq_error liq_image_set_memory_ownership(liq_image *img, int ownershi
             // for simplicity of this API there's no explicit bitmap argument,
             // so the row with the lowest address is assumed to be at the start of the bitmap
             img->pixels = img->rows[0];
-            int i;
-            for(i=1; i < img->height; i++) {
+            for(unsigned int i=1; i < img->height; i++) {
                 img->pixels = MIN(img->pixels, img->rows[i]);
             }
         }
@@ -572,10 +643,9 @@ LIQ_EXPORT liq_image *liq_image_create_custom(liq_attr *attr, liq_image_get_rgba
 LIQ_EXPORT liq_image *liq_image_create_rgba_rows(liq_attr *attr, void* rows[], int width, int height, double gamma)
 {
     if (width <= 0 || height <= 0) return NULL;
-    if (width > INT_MAX/height/16 || height > INT_MAX/width/16) return NULL;
+    if (width > INT_MAX/16/height || height > INT_MAX/16/width) return NULL;
 
-    int i;
-    for(i=0; i < height; i++) {
+    for(int i=0; i < height; i++) {
         if (!CHECK_USER_POINTER(rows+i) || !CHECK_USER_POINTER(rows[i])) return NULL;
     }
     return liq_image_create_internal(attr, (rgba_pixel**)rows, NULL, NULL, width, height, gamma);
@@ -591,8 +661,7 @@ LIQ_EXPORT liq_error liq_image_create_rgba_rows_2 (liq_attr *attr, void* rows[],
     if (width > INT_MAX/height/16 || height > INT_MAX/width/16) {
 	return LIQ_HUGE_DIMENSIONS;
     }
-    int i;
-    for(i=0; i < height; i++) {
+    for(int i=0; i < height; i++) {
         if (!CHECK_USER_POINTER(rows+i) || !CHECK_USER_POINTER(rows[i])) {
 	    return LIQ_BAD_USER_POINTER;
 	}
@@ -604,7 +673,7 @@ LIQ_EXPORT liq_error liq_image_create_rgba_rows_2 (liq_attr *attr, void* rows[],
 LIQ_EXPORT liq_image *liq_image_create_rgba(liq_attr *attr, void* bitmap, int width, int height, double gamma)
 {
     if (width <= 0 || height <= 0 || gamma < 0 || gamma > 1.0) return NULL;
-    if (width > INT_MAX/height/16 || height > INT_MAX/width/16) return NULL;
+    if (width > INT_MAX/16/height || height > INT_MAX/16/width) return NULL;
     if (!CHECK_STRUCT_TYPE(attr, liq_attr)) return NULL;
     if (!CHECK_USER_POINTER(bitmap)) return NULL;
 
@@ -612,13 +681,13 @@ LIQ_EXPORT liq_image *liq_image_create_rgba(liq_attr *attr, void* bitmap, int wi
     rgba_pixel **rows = attr->malloc(sizeof(rows[0])*height);
     if (!rows) return NULL;
 
-    int i;
-    for(i=0; i < height; i++) {
+    for(int i=0; i < height; i++) {
         rows[i] = pixels + width * i;
     }
 
     liq_image *image = liq_image_create_internal(attr, rows, NULL, NULL, width, height, gamma);
-    liq_image_set_memory_ownership(image, LIQ_OWN_ROWS);
+    image->free_rows = true;
+    image->free_rows_internal = true;
     return image;
 }
 
@@ -661,8 +730,7 @@ static void convert_row_to_f(liq_image *img, f_pixel *row_f_pixels, const unsign
 
     const rgba_pixel *const row_pixels = liq_image_get_row_rgba(img, row);
 
-    int col;
-    for(col=0; col < img->width; col++) {
+    for(unsigned int col=0; col < img->width; col++) {
         row_f_pixels[col] = to_f(gamma_lut, row_pixels[col]);
     }
 }
@@ -689,8 +757,7 @@ static const f_pixel *liq_image_get_row_f(liq_image *img, unsigned int row)
 
         float gamma_lut[256];
         to_f_set_gamma(gamma_lut, img->gamma);
-        int row;
-        for(row=0; row < img->height; row++) {
+        for(unsigned int row=0; row < img->height; row++) {
             convert_row_to_f(img, &img->f_pixels[row*img->width], row, gamma_lut);
         }
     }
@@ -709,15 +776,26 @@ LIQ_EXPORT int liq_image_get_height(const liq_image *input_image)
     return input_image->height;
 }
 
+typedef void free_func(void*);
+
+free_func *get_default_free_func(liq_image *img)
+{
+    // When default allocator is used then user-supplied pointers must be freed with free()
+    if (img->free_rows_internal || img->free != liq_aligned_free) {
+        return img->free;
+    }
+    return free;
+}
+
 static void liq_image_free_rgba_source(liq_image *input_image)
 {
     if (input_image->free_pixels && input_image->pixels) {
-        input_image->free(input_image->pixels);
+        get_default_free_func(input_image)(input_image->pixels);
         input_image->pixels = NULL;
     }
 
     if (input_image->free_rows && input_image->rows) {
-        input_image->free(input_image->rows);
+        get_default_free_func(input_image)(input_image->rows);
         input_image->rows = NULL;
     }
 }
@@ -796,7 +874,7 @@ LIQ_EXPORT liq_error liq_set_dithering_level(liq_result *res, float dither_level
     return LIQ_OK;
 }
 
-LIQ_EXPORT liq_remapping_result *liq_remap(liq_result *result, liq_image *image)
+static liq_remapping_result *liq_remapping_result_create(liq_result *result)
 {
     if (!CHECK_STRUCT_TYPE(result, liq_result)) return NULL;
 
@@ -900,8 +978,7 @@ static void sort_palette(colormap *map, const liq_attr *options)
     ** therefore be omitted from the tRNS chunk.
     */
     if (options->last_index_transparent) {
-	int i;
-	for(i=0; i < map->colors; i++) {
+	for(unsigned int i=0; i < map->colors; i++) {
 	    if (map->palette[i].acolor.a < 1.0/256.0) {
 		const unsigned int old = i, transparent_dest = map->colors-1;
 
@@ -915,8 +992,7 @@ static void sort_palette(colormap *map, const liq_attr *options)
     }
     /* move transparent colors to the beginning to shrink trns chunk */
     unsigned int num_transparent=0;
-    int i;
-    for(i=0; i < map->colors; i++) {
+    for(unsigned int i=0; i < map->colors; i++) {
         if (map->palette[i].acolor.a < 255.0/256.0) {
             // current transparent color is swapped with earlier opaque one
             if (i != num_transparent) {
@@ -944,7 +1020,8 @@ static void sort_palette(colormap *map, const liq_attr *options)
     }
 }
 
-inline static unsigned int posterize_channel(unsigned int color, unsigned int bits) {
+inline static unsigned int posterize_channel(unsigned int color, unsigned int bits)
+{
     return (color & ~((1<<bits)-1)) | (color >> (8-bits));
 }
 
@@ -954,8 +1031,7 @@ static void set_rounded_palette(liq_palette *const dest, colormap *const map, co
     to_f_set_gamma(gamma_lut, gamma);
 
     dest->count = map->colors;
-    int x;
-    for(x = 0; x < map->colors; ++x) {
+    for(unsigned int x = 0; x < map->colors; ++x) {
         rgba_pixel px = to_rgb(gamma, map->palette[x].acolor);
 
         px.r = posterize_channel(px.r, posterize);
@@ -1004,12 +1080,10 @@ static float remap_to_palette(liq_image *const input_image, unsigned char *const
     #pragma omp parallel for if (rows*cols > 3000) \
         default(none) shared(average_color) reduction(+:remapping_error)
 #endif /* def _OPENMP */
-    int row;
-    for(row = 0; row < rows; ++row) {
+    for(int row = 0; row < rows; ++row) {
         const f_pixel *const row_pixels = liq_image_get_row_f(input_image, row);
         unsigned int last_match=0;
-        int col;
-        for(col = 0; col < cols; ++col) {
+        for(unsigned int col = 0; col < cols; ++col) {
             f_pixel px = row_pixels[col];
             float diff;
 
@@ -1049,15 +1123,15 @@ inline static f_pixel get_dithered_pixel(const float dither_level, const float m
          if (a > 1.0) a = 1.0;
     else if (a < 0)   a = 0;
 
-     // If dithering error is crazy high, don't propagate it that much
-     // This prevents crazy geen pixels popping out of the blue (or red or black! ;)
-     const float dither_error = sr*sr + sg*sg + sb*sb + sa*sa;
-     if (dither_error > max_dither_error) {
-         ratio *= 0.8;
-     } else if (dither_error < 2.f/256.f/256.f) {
+    // If dithering error is crazy high, don't propagate it that much
+    // This prevents crazy geen pixels popping out of the blue (or red or black! ;)
+    const float dither_error = sr*sr + sg*sg + sb*sb + sa*sa;
+    if (dither_error > max_dither_error) {
+        ratio *= 0.8;
+    } else if (dither_error < 2.f/256.f/256.f) {
         // don't dither areas that don't have noticeable error — makes file smaller
         return px;
-     }
+    }
 
      return (f_pixel){
          .r=px.r + sr * ratio,
@@ -1089,8 +1163,7 @@ static void remap_to_palette_floyd(liq_image *input_image, unsigned char *const
     srand(12345); /* deterministic dithering is better for comparing results */
     if (!thiserr) return;
 
-    int col;
-    for (col = 0; col < cols + 2; ++col) {
+    for (unsigned int col = 0; col < cols + 2; ++col) {
         const double rand_max = RAND_MAX;
         thiserr[col].r = ((double)rand() - rand_max/2.0)/rand_max/255.0;
         thiserr[col].g = ((double)rand() - rand_max/2.0)/rand_max/255.0;
@@ -1108,8 +1181,7 @@ static void remap_to_palette_floyd(liq_image *input_image, unsigned char *const
 
     bool fs_direction = true;
     unsigned int last_match=0;
-    int row;
-    for (row = 0; row < rows; ++row) {
+    for (unsigned int row = 0; row < rows; ++row) {
         memset(nexterr, 0, (cols + 2) * sizeof(*nexterr));
 
         unsigned int col = (fs_direction) ? 0 : (cols - 1);
@@ -1196,8 +1268,7 @@ static void remap_to_palette_floyd(liq_image *input_image, unsigned char *const
                 if (col <= 0) break;
                 --col;
             }
-        }
-        while(1);
+        } while(1);
 
         f_pixel *const temperr = thiserr;
         thiserr = nexterr;
@@ -1236,8 +1307,7 @@ static histogram *get_histogram(liq_image *input_image, liq_attr *options)
 
         // histogram uses noise contrast map for importance. Color accuracy in noisy areas is not very important.
         // noise map does not include edges to avoid ruining anti-aliasing
-        int row;
-        for(row=0; row < rows; row++) {
+        for(unsigned int row=0; row < rows; row++) {
             bool added_ok;
             if (all_rows_at_once) {
                 added_ok = pam_computeacolorhash(acht, (const rgba_pixel *const *)input_image->rows, cols, rows, input_image->noise);
@@ -1288,8 +1358,7 @@ static void modify_alpha(liq_image *input_image, rgba_pixel *const row_pixels)
     const float almost_opaque_val = min_opaque_val * 169.f/256.f;
     const unsigned int almost_opaque_val_int = (min_opaque_val * 169.f/256.f)*255.f;
 
-    int col;
-    for(col = 0; col < input_image->width; col++) {
+    for(unsigned int col = 0; col < input_image->width; col++) {
         const rgba_pixel px = row_pixels[col];
 
         /* ie bug: to avoid visible step caused by forced opaqueness, linearily raise opaqueness of almost-opaque colors */
@@ -1310,26 +1379,28 @@ static void modify_alpha(liq_image *input_image, rgba_pixel *const row_pixels)
 static void contrast_maps(liq_image *image)
 {
     const int cols = image->width, rows = image->height;
-    if (cols < 4 || rows < 4 || (3*cols*rows) > LIQ_HIGH_MEMORY_LIMIT) return;
+    if (cols < 4 || rows < 4 || (3*cols*rows) > LIQ_HIGH_MEMORY_LIMIT) {
+        return;
+    }
 
     unsigned char *restrict noise = image->malloc(cols*rows);
     unsigned char *restrict edges = image->malloc(cols*rows);
     unsigned char *restrict tmp = image->malloc(cols*rows);
 
-    if (!noise || !edges || !tmp) return;
+    if (!noise || !edges || !tmp) {
+        return;
+    }
 
     const f_pixel *curr_row, *prev_row, *next_row;
     curr_row = prev_row = next_row = liq_image_get_row_f(image, 0);
 
-    int j;
-    for (j=0; j < rows; j++) {
+    for (int j=0; j < rows; j++) {
         prev_row = curr_row;
         curr_row = next_row;
         next_row = liq_image_get_row_f(image, MIN(rows-1,j+1));
 
         f_pixel prev, curr = curr_row[0], next=curr;
-        int i;
-        for (i=0; i < cols; i++) {
+        for (int i=0; i < cols; i++) {
             prev=curr;
             curr=next;
             next = curr_row[MIN(cols-1,i+1)];
@@ -1377,8 +1448,7 @@ static void contrast_maps(liq_image *image)
 
     min3(edges, tmp, cols, rows);
     max3(tmp, edges, cols, rows);
-    int i;
-    for(i=0; i < cols*rows; i++) edges[i] = MIN(noise[i], edges[i]);
+    for(int i=0; i < cols*rows; i++) edges[i] = MIN(noise[i], edges[i]);
 
     image->free(tmp);
 
@@ -1399,13 +1469,11 @@ static void update_dither_map(unsigned char *const *const row_pointers, liq_imag
     const unsigned int height = input_image->height;
     unsigned char *const edges = input_image->edges;
 
-    int row;
-    for(row=0; row < height; row++) {
+    for(unsigned int row=0; row < height; row++) {
         unsigned char lastpixel = row_pointers[row][0];
         unsigned int lastcol=0;
 
-        int col;
-        for(col=1; col < width; col++) {
+        for(unsigned int col=1; col < width; col++) {
             const unsigned char px = row_pointers[row][col];
 
             if (px != lastpixel || col == width-1) {
@@ -1460,7 +1528,9 @@ static colormap *find_best_palette(histogram *hist, const liq_attr *options, dou
     const double percent = (double)(feedback_loop_trials>0?feedback_loop_trials:1)/100.0;
 
     do {
-        colormap *newmap = mediancut(hist, options->min_opaque_val, max_colors, target_mse * target_mse_overshoot, MAX(MAX(90.0/65536.0, target_mse), least_error)*1.2);
+        colormap *newmap = mediancut(hist, options->min_opaque_val, max_colors,
+            target_mse * target_mse_overshoot, MAX(MAX(90.0/65536.0, target_mse), least_error)*1.2,
+            options->malloc, options->free);
 
         if (feedback_loop_trials <= 0) {
             return newmap;
@@ -1492,8 +1562,7 @@ static colormap *find_best_palette(histogram *hist, const liq_attr *options, dou
 
             feedback_loop_trials -= 1; // asymptotic improvement could make it go on forever
         } else {
-            int j;
-            for(j=0; j < hist->size; j++) {
+            for(unsigned int j=0; j < hist->size; j++) {
                 hist->achv[j].adjusted_weight = (hist->achv[j].perceptual_weight + hist->achv[j].adjusted_weight)/2.0;
             }
 
@@ -1510,8 +1579,7 @@ static colormap *find_best_palette(histogram *hist, const liq_attr *options, dou
 
     // likely_colormap_index (used and set in viter_do_iteration) can't point to index outside colormap
     if (acolormap->colors < 256) {
-	int j;
-	for(j=0; j < hist->size; j++) {
+	for(unsigned int j=0; j < hist->size; j++) {
 	    if (hist->achv[j].likely_colormap_index >= acolormap->colors) {
 		hist->achv[j].likely_colormap_index = 0; // actual value doesn't matter, as the guess is out of date anyway
 	    }
@@ -1529,9 +1597,8 @@ static liq_result *pngquant_quantize(histogram *hist, const liq_attr *options, c
     // If image has few colors to begin with (and no quality degradation is required)
     // then it's possible to skip quantization entirely
     if (hist->size <= options->max_colors && options->target_mse == 0) {
-        acolormap = pam_colormap(hist->size);
-        int i;
-        for(i=0; i < hist->size; i++) {
+        acolormap = pam_colormap(hist->size, options->malloc, options->free);
+        for(unsigned int i=0; i < hist->size; i++) {
             acolormap->palette[i].acolor = hist->achv[i].acolor;
             acolormap->palette[i].popularity = hist->achv[i].perceptual_weight;
         }
@@ -1551,8 +1618,7 @@ static liq_result *pngquant_quantize(histogram *hist, const liq_attr *options, c
 
             double previous_palette_error = MAX_DIFF;
 
-            int i;
-            for(i=0; i < iterations; i++) {
+            for(unsigned int i=0; i < iterations; i++) {
                 palette_error = viter_do_iteration(hist, acolormap, options->min_opaque_val, NULL, i==0 || options->fast_palette);
 
                 if (fabs(previous_palette_error-palette_error) < iteration_limit) {
@@ -1608,8 +1674,7 @@ LIQ_EXPORT liq_error liq_write_remapped_image(liq_result *result, liq_image *inp
 
     unsigned char *rows[input_image->height];
     unsigned char *buffer_bytes = buffer;
-    int i;
-    for(i=0; i < input_image->height; i++) {
+    for(unsigned int i=0; i < input_image->height; i++) {
         rows[i] = &buffer_bytes[input_image->width * i];
     }
     return liq_write_remapped_image_rows(result, input_image, rows);
@@ -1619,15 +1684,14 @@ LIQ_EXPORT liq_error liq_write_remapped_image_rows(liq_result *quant, liq_image
 {
     if (!CHECK_STRUCT_TYPE(quant, liq_result)) return LIQ_INVALID_POINTER;
     if (!CHECK_STRUCT_TYPE(input_image, liq_image)) return LIQ_INVALID_POINTER;
-    int i;
-    for(i=0; i < input_image->height; i++) {
+    for(unsigned int i=0; i < input_image->height; i++) {
         if (!CHECK_USER_POINTER(row_pointers+i) || !CHECK_USER_POINTER(row_pointers[i])) return LIQ_INVALID_POINTER;
     }
 
     if (quant->remapping) {
         liq_remapping_result_destroy(quant->remapping);
     }
-    liq_remapping_result *const result = quant->remapping = liq_remap(quant, input_image);
+    liq_remapping_result *const result = quant->remapping = liq_remapping_result_create(quant);
     if (!result) return LIQ_OUT_OF_MEMORY;
 
     if (!input_image->edges && !input_image->dither_map && quant->use_dither_map) {
